@@ -2,7 +2,7 @@
 /*
   Copyright(C) 2010 Tetsuro IKEDA
   Copyright(C) 2010-2013 Kentoku SHIBA
-  Copyright(C) 2011-2019 Kouhei Sutou <kou@clear-code.com>
+  Copyright(C) 2011-2020 Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -68,7 +68,6 @@ extern "C" {
 #  define MRN_HANDLER_HAVE_HA_PREPARE_INPLACE_ALTER_TABLE 1
 #  define MRN_HANDLER_HAVE_HA_INPLACE_ALTER_TABLE 1
 #  define MRN_HANDLER_HAVE_HA_COMMIT_INPLACE_ALTER_TABLE 1
-#  define MRN_SUPPORT_FOREIGN_KEYS 1
 #endif
 
 #ifndef MRN_MARIADB_P
@@ -443,6 +442,11 @@ typedef uint mrn_alter_table_flags;
 
 #ifdef MRN_PERCONA_P
 #  define MRN_HANDLER_HAVE_HAS_GAP_LOCKS
+#  if MYSQL_VERSION_ID >= 80018
+#    define MRN_HANDLER_HAS_GAP_LOCKS_NOEXCEPT noexcept
+#  else
+#    define MRN_HANDLER_HAS_GAP_LOCKS_NOEXCEPT
+#  endif
 #endif
 
 #if MYSQL_VERSION_ID < 80013 || defined(MRN_MARIADB_P)
@@ -476,6 +480,10 @@ typedef uint mrn_alter_table_flags;
 #  define MRN_HANDLER_PRIMARY_KEY_IS_CLUSTERED_CONST
 #endif
 
+#if MYSQL_VERSION_ID < 80019 || defined(MRN_MARIADB_P)
+#  define MRN_HANDLER_HAVE_FOREIGN_KEY_INFO
+#endif
+
 class ha_mroonga;
 
 /* structs */
@@ -875,7 +883,7 @@ public:
   int start_stmt(THD *thd, thr_lock_type lock_type);
 
 #ifdef MRN_HANDLER_HAVE_HAS_GAP_LOCKS
-  bool has_gap_locks() const mrn_override;
+  bool has_gap_locks() const MRN_HANDLER_HAS_GAP_LOCKS_NOEXCEPT mrn_override;
 #endif
 
 protected:
@@ -910,19 +918,23 @@ protected:
   bool primary_key_is_clustered()
     MRN_HANDLER_PRIMARY_KEY_IS_CLUSTERED_CONST
     mrn_override;
-  bool is_fk_defined_on_table_or_index(uint index);
-  char *get_foreign_key_create_info();
 #ifdef MRN_HANDLER_HAVE_GET_TABLESPACE_NAME
   char *get_tablespace_name(THD *thd, char *name, uint name_len);
 #endif
 #ifdef MRN_HANDLER_HAVE_CAN_SWITCH_ENGINES
   bool can_switch_engines();
 #endif
-  int get_foreign_key_list(THD *thd, List<FOREIGN_KEY_INFO> *f_key_list);
-  int get_parent_foreign_key_list(THD *thd, List<FOREIGN_KEY_INFO> *f_key_list);
-  uint referenced_by_foreign_key();
+#ifdef MRN_HANDLER_HAVE_FOREIGN_KEY_INFO
+  bool is_fk_defined_on_table_or_index(uint index) mrn_override;
+  char *get_foreign_key_create_info() mrn_override;
+  int get_foreign_key_list(THD *thd,
+                           List<FOREIGN_KEY_INFO> *f_key_list) mrn_override;
+  int get_parent_foreign_key_list(THD *thd,
+                                  List<FOREIGN_KEY_INFO> *f_key_list) mrn_override;
+  uint referenced_by_foreign_key()mrn_override;
+  void free_foreign_key_create_info(char* str) mrn_override;
+#endif
   void init_table_handle_for_HANDLER();
-  void free_foreign_key_create_info(char* str);
 #ifdef MRN_HAVE_HA_REBIND_PSI
   void unbind_psi();
   void rebind_psi();
@@ -1792,17 +1804,13 @@ private:
   int wrapper_start_stmt(THD *thd, thr_lock_type lock_type);
   int storage_start_stmt(THD *thd, thr_lock_type lock_type);
 #ifdef MRN_HANDLER_HAVE_HAS_GAP_LOCKS
-  bool wrapper_has_gap_locks() const;
-  bool storage_has_gap_locks() const;
+  bool wrapper_has_gap_locks() const MRN_HANDLER_HAS_GAP_LOCKS_NOEXCEPT;
+  bool storage_has_gap_locks() const MRN_HANDLER_HAS_GAP_LOCKS_NOEXCEPT;
 #endif
   bool wrapper_primary_key_is_clustered()
     MRN_HANDLER_PRIMARY_KEY_IS_CLUSTERED_CONST;
   bool storage_primary_key_is_clustered()
     MRN_HANDLER_PRIMARY_KEY_IS_CLUSTERED_CONST;
-  bool wrapper_is_fk_defined_on_table_or_index(uint index);
-  bool storage_is_fk_defined_on_table_or_index(uint index);
-  char *wrapper_get_foreign_key_create_info();
-  char *storage_get_foreign_key_create_info();
 #ifdef MRN_HANDLER_HAVE_GET_TABLESPACE_NAME
   char *wrapper_get_tablespace_name(THD *thd, char *name, uint name_len);
   char *storage_get_tablespace_name(THD *thd, char *name, uint name_len);
@@ -1811,16 +1819,22 @@ private:
   bool wrapper_can_switch_engines();
   bool storage_can_switch_engines();
 #endif
+#ifdef MRN_HANDLER_HAVE_FOREIGN_KEY_INFO
+  bool wrapper_is_fk_defined_on_table_or_index(uint index);
+  bool storage_is_fk_defined_on_table_or_index(uint index);
+  char *wrapper_get_foreign_key_create_info();
+  char *storage_get_foreign_key_create_info();
   int wrapper_get_foreign_key_list(THD *thd, List<FOREIGN_KEY_INFO> *f_key_list);
   int storage_get_foreign_key_list(THD *thd, List<FOREIGN_KEY_INFO> *f_key_list);
   int wrapper_get_parent_foreign_key_list(THD *thd, List<FOREIGN_KEY_INFO> *f_key_list);
   int storage_get_parent_foreign_key_list(THD *thd, List<FOREIGN_KEY_INFO> *f_key_list);
   uint wrapper_referenced_by_foreign_key();
   uint storage_referenced_by_foreign_key();
-  void wrapper_init_table_handle_for_HANDLER();
-  void storage_init_table_handle_for_HANDLER();
   void wrapper_free_foreign_key_create_info(char* str);
   void storage_free_foreign_key_create_info(char* str);
+#endif
+  void wrapper_init_table_handle_for_HANDLER();
+  void storage_init_table_handle_for_HANDLER();
   void wrapper_set_keys_in_use();
   void storage_set_keys_in_use();
 #ifdef MRN_RBR_UPDATE_NEED_ALL_COLUMNS
